/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "spds/SyncItem.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "syncml/core/TagNames.h"
#include "base/util/ArrayListEnumeration.h"
#include "customization.h"
#include "pim/PicturesSyncSource.h"
#include "pim/ClientSettings.h"
#include "pim/Utils.h"
#include "base/stat.h"

BEGIN_NAMESPACE

/**
 * Fills and returns a MediaSyncSourceParams object, with current informations
 * about Server URL, username and Client sw version.
 * These params are needed by MediaSyncSource to correctly handle the cache file.
 */
MediaSyncSourceParams getMediaParams() 
{
    ClientSettings* config = ClientSettings::getInstance();
    MediaSyncSourceParams params;
    
    params.setUrl      (config->getSyncURL());
    params.setUsername (config->getUsername());
    params.setSwv      (config->getSwv());

    // Set the filtering by size (exclude pics with size > MAX_IMAGE_SIZE)
    params.setFilterBySize(MAX_IMAGE_SIZE);
    
    // Set the filtering by date (exclude pics modified BEFORE install timestamp)
    WindowsSyncSourceConfig* wssc = config->getWinSyncSourceConfig(SOURCE_NAME_PICTURE);
    if (wssc) {
        if (wssc->getIncludeOldItems() == false) {
            params.setFilterByDate(config->getInstallTimestamp());   // it's in UTC
        }
    }

    return params;
}


PicturesSyncSource::PicturesSyncSource(const WCHAR* name, 
                                       AbstractSyncSourceConfig* sc, 
                                       const StringBuffer& aDir)
                                       : MediaSyncSource(name, sc, aDir, getMediaParams())
{
    // Force to recurse into subfolders.
    setRecursive(false);

}

bool PicturesSyncSource::filterOutgoingItem(const StringBuffer& fullName, struct stat& st)
{
    // [removed] Skip images bigger than MAX_IMAGE_SIZE 
    // (it's done in MediaSyncSource)
    

    if (! S_ISDIR(st.st_mode) ) {
        // Skip non-image files
        if ( !checkFileExtension(fullName, "jpg",  true) &&
             !checkFileExtension(fullName, "jpeg", true) &&
             !checkFileExtension(fullName, "gif",  true) &&
             !checkFileExtension(fullName, "png",  true) ) {
            // not an image file
            return true;
        }
    }
    
    return MediaSyncSource::filterOutgoingItem(fullName, st);
}


END_NAMESPACE
