/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */



#include "pim/WindowsSyncSourceConfig.h"
#include "base/util/utils.h"
#include "syncml/core/CTCap.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/UtilsRecurrence.h"

WindowsSyncSourceConfig::WindowsSyncSourceConfig() {

    initialize();
    // Please note that 's' pointer MUST be set!!
}

WindowsSyncSourceConfig::WindowsSyncSourceConfig(SyncSourceConfig* sc) {

    initialize();

    // Link SyncSourceConfig pointer with common properties:
    if (!sc) {
        setError( ERR_NULLPTR_WINDSYNCSOURCECONFIG , "WindowsSyncSourceConfig initialized with a NULL pointer!" );
        throw getLastErrorMsg();
    }
    s = sc;
    populateCTCap();
}

WindowsSyncSourceConfig::~WindowsSyncSourceConfig() {    
    s = NULL;
}


// Return the pointer to external SyncSourceConfig object
// used for common properties.
SyncSourceConfig* WindowsSyncSourceConfig::getCommonConfig() {
    return s;
}

/*
 * Copy Constructor
 */
WindowsSyncSourceConfig::WindowsSyncSourceConfig(const WindowsSyncSourceConfig& wsc) {

    if (&wsc == this) {
        return;
    }

    initialize();

    // WARNING! pointer to the same object!
    s = wsc.s;

    populateCTCap();  
}


/*
 * Operator =
 */
WindowsSyncSourceConfig& WindowsSyncSourceConfig::operator = (const WindowsSyncSourceConfig& wsc) {

    initialize();

    // WARNING! pointer to the same object!
    s = wsc.s;

    populateCTCap();

    return *this;
}



/**
 * Assign the internal SyncSourceConfig* pointer.
 */
void WindowsSyncSourceConfig::setCommonConfig(SyncSourceConfig* sc) {
    s = sc;
    populateCTCap();
}



// Initialize members
void WindowsSyncSourceConfig::initialize() {

    s = NULL;   
    firstSyncTime = 0;
    includeOldItems = true;
}

__declspec(dllexport) const string& WindowsSyncSourceConfig::getSyncDir() const {
    return dir;
}

const string& WindowsSyncSourceConfig::getUseSif()  const  {
    return useSif;
}

void WindowsSyncSourceConfig::populateCTCap(){
    if (strcmp ( s->getName(), "contact" ) == 0){
        //adding sif
        ArrayList* p = getContactProperties();
        s->addCtCap(p);
        delete p;

        //adding vcard
        p = getVCardProperties();
        s->addCtCap(p, "text/x-vcard", "2.1");
        delete p;
    }
    if (strcmp ( s->getName(), "calendar" ) == 0){
        //adding sif
        ArrayList* p = getAppointmentProperties();
        p->add(getRecurrenceProperties());
        Property prop;
        prop.setPropName("Exceptions");
        p->add(prop);
        s->addCtCap(p);
        delete p;

        //adding vcalendar
        p = getVCalendarProperties();
        s->addCtCap(p, "text/x-vcalendar", "1.0");
        delete p;
    }
    if (strcmp ( s->getName(), "task" ) == 0){
        //adding sif
        ArrayList* p = getTaskProperties();
        p->add(getRecurrenceProperties());
        s->addCtCap(p);
        delete p;
        //adding vTodo
        p = getVTodoProperties();
        s->addCtCap(p, "text/x-vcalendar", "1.0");
        delete p;
    }
    if (strcmp ( s->getName(), "note" ) == 0){
        ArrayList* p = new ArrayList();
        
        //adding sif
        Property subject,body;
        subject.setPropName("Subject");
        body.setPropName("Body");
        p->add(subject);
        p->add(body);
        s->addCtCap(p);

        p->clear();
        
        //adding vNote
        Property pval;
        subject.setPropName("SUBJECT");
        body.setPropName("BODY");
        ArrayList valenums;
        StringBuffer val;

        pval.setPropName("BEGIN");
        val = "VNOTE";
        valenums.add(val);
        pval.setValEnums(&valenums);
        p->add(pval);

        valenums.clear();

        pval.setPropName("END");
        val = "VNOTE";
        valenums.add(val);
        pval.setValEnums(&valenums);
        p->add(pval);

        valenums.clear();

        pval.setPropName("VERSION");
        val = "1.1";
        valenums.add(val);
        pval.setValEnums(&valenums);
        p->add(pval);
        p->add(subject);
        p->add(body);
        s->addCtCap(p, "text/x-vnote", "1.1");
        delete p;
    }
}

StringBuffer WindowsSyncSourceConfig::print() {

    StringBuffer ret;
    ret = "** SOURCE: "; ret += getName(); ret += "**\r\n";
    
    ret += "URI:\t\t"; ret += getURI(); ret += "\r\n";
    ret += "SyncModes:\t"; ret += getSyncModes(); ret += "\r\n";
    ret += "Type:\t\t"; ret += getType(); ret += "\r\n";
    ret += "Sync:\t\t"; ret += getSync(); ret += "\r\n";
    ret += "Encoding:\t"; ret += getEncoding(); ret += "\r\n";
    ret += "Version:\t"; ret += getVersion(); ret += "\r\n";
    ret += "SupportedType:\t"; ret += getSupportedTypes(); ret += "\r\n";
    ret += "Last:\t\t"; ret.append(getLast()); ret += "\r\n";
    ret += "Encryption:\t"; ret += getEncryption(); ret += "\r\n";    
    ret += "SyncDir:\t"; ret += getSyncDir().c_str(); ret += "\r\n";
    ret += "UseSif:\t\t"; ret += getUseSif().c_str(); ret += "\r\n";
    ret += "Enabled:\t"; ret += (isEnabled() == true ? "1" : "0"); ret += "\r\n";    

            
   return ret;
}

