/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "pim/mail/AccountProvisioner.h"
#include "mail/MailAccount.h"
#include "base/util/StringBuffer.h"


#include <windows.h>
#include <stdio.h>
#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>
#include "Cfgmgrapi.h" //-- for DMProcessConfigXML


// The default values used, if missing
#define DEFAULT_PROTOCOL    "SyncML"
#define DEFAULT_IN_SERVER   "sync.funambol.com"
#define DEFAULT_OUT_SERVER  "sync.funambol.com"
#define DEFAULT_DOMAIN_NAME "funambol.com"
#define DEFAULT_USERNAME    ""
#define DEFAULT_PASSWORD    ""


#define CREATE_ACCOUNT "<wap-provisioningdoc> \
 <characteristic type=\"EMAIL2\"> \
 <characteristic type=\"%s\"> \
 <parm name=\"SERVICENAME\" value=\"%s\"/> \
 <parm name=\"SERVICETYPE\" value=\"%s\"/> \
 <parm name=\"INSERVER\"    value=\"%s\"/> \
 <parm name=\"OUTSERVER\"   value=\"%s\"/> \
 <parm name=\"AUTHNAME\"    value=\"%s\"/> \
 <parm name=\"AUTHSECRET\"  value=\"%s\"/> \
 <parm name=\"DOMAIN\"      value=\"%s\"/> \
 <parm name=\"REPLYADDR\"   value=\"%s\"/> \
 <parm name=\"NAME\"        value=\"%s\"/> \
 <parm name=\"LINGER\"      value=\"0\"/> \
 </characteristic> \
 </characteristic> \
 </wap-provisioningdoc>"

#define UPDATE_ACCOUNT "<wap-provisioningdoc> \
 <characteristic type=\"EMAIL2\"> \
 <characteristic type=\"%s\"> \
 <parm name=\"INSERVER\"    value=\"%s\"/> \
 <parm name=\"OUTSERVER\"   value=\"%s\"/> \
 <parm name=\"AUTHNAME\"    value=\"%s\"/> \
 <parm name=\"AUTHSECRET\"  value=\"%s\"/> \
 <parm name=\"DOMAIN\"      value=\"%s\"/> \
 <parm name=\"REPLYADDR\"   value=\"%s\"/> \
 <parm name=\"NAME\"        value=\"%s\"/> \
 <parm name=\"LINGER\"      value=\"0\"/> \
 </characteristic> \
 </characteristic> \
 </wap-provisioningdoc>"

#define DELETE_ACCOUNT L"<wap-provisioningdoc><characteristic type=\"EMAIL2\"> \
 <nocharacteristic type=\"%s\"/> \
 </characteristic></wap-provisioningdoc>"

#define ACCOUNT_QUERY L"<wap-provisioningdoc>\
 <characteristic type=\"EMAIL2\">\
 <characteristic-query type=\"%s\"/>\
 </characteristic>\
 </wap-provisioningdoc>"


USE_NAMESPACE

wstring AccountProvisioner::createGUID() const {
   
    GUID pGuid;
    CoCreateGuid(&pGuid);
    return formatGUID(&pGuid);
}

// return the guid is MS format
wstring AccountProvisioner::formatGUID(GUID* pGuid) const {

    wstring ret;
    wchar_t guidW[64];
    memset(guidW, 0, 64*sizeof(wchar_t));
    wsprintf(guidW,
        TEXT("{%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X}"),
        pGuid->Data1, pGuid->Data2, pGuid->Data3,
        pGuid->Data4[0], pGuid->Data4[1],
        pGuid->Data4[2], pGuid->Data4[3],
        pGuid->Data4[4], pGuid->Data4[5],
        pGuid->Data4[6], pGuid->Data4[7]);
    
    return ret.append(guidW);
}


int AccountProvisioner::addAccount(MailAccount& account, wstring& accountGuid) {
    
    // Returned to caller
    accountGuid  = createGUID();
    
    LPWSTR out = NULL;
	const char* name         =   account.getName();
    const char* type         =   account.getProtocol();
    const char* inserver     =   account.getInServer();
    const char* outserver    =   account.getOutServer();
    const char* username     =   account.getUsername();
    const char* password     =   account.getPassword();
    const char* domain       =   account.getDomainName();
    const char* address      =   account.getEmailAddress();
    const char* visibleName  =   account.getVisibleName();

    // Name and address are mandatory 
    if ( name == NULL        || (strlen(name)        == 0) ||
         address == NULL     || (strlen(address)     == 0) ) {
        return E_FAIL;
    }

    StringBuffer accountProvisionString;
    const char* guid = toMultibyte(accountGuid.c_str());

    accountProvisionString.sprintf(CREATE_ACCOUNT, guid, name, 
        type ? type : DEFAULT_PROTOCOL, 
        inserver ? inserver : DEFAULT_IN_SERVER, 
        outserver ? outserver : DEFAULT_OUT_SERVER, 
        username ? username : DEFAULT_USERNAME, 
        password ? password : DEFAULT_PASSWORD, 
        domain ? domain : DEFAULT_DOMAIN_NAME, 
        address, visibleName ? visibleName : "");
    
    const wchar_t* accountString = toWideChar(accountProvisionString.c_str());
    HRESULT hr = DMProcessConfigXML((LPCWSTR)accountString, CFGFLAG_PROCESS, &out);

    delete [] guid;
    delete [] accountString;

    return hr;
}


// we suppose that the server sends us all the fields of the email account
// for account updates we can not modify account type (IMAP, POP, SYNCML)
// and its name
int AccountProvisioner::modAccount(const MailAccount& account, const WCHAR* accountGuid) {
    
    if (!accountGuid || !wcslen(accountGuid)) {
        LOG.error("Error: empty account GUID");
        return -1;
    }

    LPWSTR out = NULL;
    const char* inserver     =   account.getInServer();
    const char* outserver    =   account.getOutServer();
    const char* username     =   account.getUsername();
    const char* password     =   account.getPassword();
    const char* domain       =   account.getDomainName();
    const char* address      =   account.getEmailAddress();
    const char* visiblename  =   account.getVisibleName();
    const char* guid         =   toMultibyte(accountGuid);
    StringBuffer updateAccountFmt;

    updateAccountFmt.sprintf(UPDATE_ACCOUNT, guid, 
        inserver ? inserver : "", outserver ? outserver : "", 
        username ? username : "", password ? password : "", 
        domain ? domain : "", address ? address : "", 
        visiblename ? visiblename : "");
                            
    const wchar_t* updateAccountCmd = toWideChar(updateAccountFmt.c_str()); 
    HRESULT hr = DMProcessConfigXML((LPCWSTR)updateAccountCmd, CFGFLAG_PROCESS, &out);

    delete [] guid;
    delete [] updateAccountCmd;

    return hr;
}

int AccountProvisioner::delAccount(const WCHAR* accountGuid){

    if (!accountGuid || !wcslen(accountGuid)) {
        LOG.error("Error: empty account GUID");
        return -1;
    }

    LPWSTR out = NULL;
    wchar_t* accountString = new wchar_t[wcslen(DELETE_ACCOUNT) + wcslen(accountGuid) + 1];
    wsprintf(accountString, DELETE_ACCOUNT, accountGuid);
    
    HRESULT hr = DMProcessConfigXML((LPCWSTR)accountString, CFGFLAG_PROCESS, &out);
    
    if (accountString) {
        delete [] accountString;
    }
    return hr;
}



