/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/Log.h"
#include "base/messages.h"
#include "base/util/utils.h"
#include "http/constants.h"
#include "http/errors.h"
#include "base/util/StringBuffer.h"
#include "HttpTransport.h"

#include "localizationUtils.h"
#include "HwndFunctions.h"

#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif

#ifdef _WIN32_WCE
#include "http/GPRSConnection.h"
#endif
USE_NAMESPACE

const int readBufferSize    = 5000;
char* bufferA               = new char[readBufferSize+1];
HINTERNET request           = NULL; 
DWORD totalLenght           = 0;
HANDLE pConnection          = NULL;
DWORD WINAPI readWorker(LPVOID lpv);
int attachSize              = 0;

char* tmpFile = mkTempFileName("attachTmp");

FILE *f                     = NULL;
int written                 = 0;
ULONG        cbWritten      = 0;
HWND h = NULL;
/**
 * Constructor.
 * In this implementation newProxy is ignored, since proxy configuration
 * is taken from the WinInet subsystem.
 *
 * @param url    the url where messages will be sent with sendMessage()
 * @param proxy  proxy information or NULL if no proxy should be used
 * @param pmsg   the message interface where to download the attachment
 */

HttpTransport::HttpTransport(URL& newURL, Proxy& newProxy, IStream* newpStream,  int size) {
    url = newURL;
    proxy.setProxy(newProxy);
    pStream = newpStream;
    realResponseLenght = 0;
    exitVar = false;
    extimatedSize = attachSize = size;

#ifdef _WIN32_WCE
    // used by default. check connection before...
    if (!EstablishConnection()) {
        LOG.error("HttpTransport: not possible to establish a connection");
    }
#endif

}

HttpTransport::~HttpTransport() {}


bool HttpTransport::downloadAttachment() {
    
    bool ret = false;    
    int status = -1;
    unsigned int contentLength = 0;
    WCHAR* wurlHost      = NULL;
    WCHAR* wurlResource  = NULL;
    char userAgent[128];
       
    if (extimatedSize > 0) {
        h = FindWindow(NULL, getLocalizationUtils()->getLocalizationString(IDS_DOWNLOADING_TITLE));
        SendMessage(h, ID_MYMSG_SET_RANGE_PROGRESS, 0, (unsigned int)(extimatedSize/readBufferSize));                          
        SendMessage(h, ID_MYMSG_GO_STEP_PROGRESS, 1, 0); 
        
    }

    HINTERNET inet       = NULL,
              connection = NULL;
              
    
    if (!(url.host) || strlen(url.host) == 0) {
        LOG.error("HttpTransport: URL not valid");
        goto exit;
    }

    DWORD size  = 0,
          read  = 0,
          flags = INTERNET_FLAG_RELOAD |
                  INTERNET_FLAG_NO_CACHE_WRITE |
                  INTERNET_FLAG_KEEP_CONNECTION |           // This is necessary if authentication is required.
                  INTERNET_FLAG_NO_COOKIES;                 // This is used to avoid possible server errors on successive sessions.

	LPCWSTR acceptTypes[2] = {TEXT("*/*"), NULL};


    // Set flags for secure connection (https).
    if (url.isSecure()) {
        flags = flags
              | INTERNET_FLAG_SECURE
              | INTERNET_FLAG_IGNORE_CERT_CN_INVALID
              | INTERNET_FLAG_IGNORE_CERT_DATE_INVALID
              ;
    }
    
    //
    // Open Internet connection.
    //
    WCHAR* ua = toWideChar(userAgent);
    inet = InternetOpen (ua, INTERNET_OPEN_TYPE_PRECONFIG, NULL, 0, 0);
	if (ua) {delete [] ua; ua = NULL; }

    if (!inet) {
        LOG.error("HttpTransport: InternetOpen Error");
        goto exit;
    }
    LOG.debug("HttpTransport: Connecting to %s:%d", url.host, url.port);

    //
    // Open an HTTP session for a specified site by using lpszServer.
    //
    wurlHost = toWideChar(url.host);
    if (!(connection = InternetConnect (inet,
                                        wurlHost,
                                        url.port,
                                        NULL, // username
                                        NULL, // password
                                        INTERNET_SERVICE_HTTP,
                                        0,
                                        0))) {
        LOG.error("HttpTransport: InternetConnect error");
        goto exit;
    }
    LOG.debug("HttpTransport: Requesting resource %s", url.resource);
    
    wurlResource = toWideChar(url.resource);
    if (!(request = HttpOpenRequest (connection,
                                     METHOD_POST,
                                     wurlResource,
                                     HTTP_VERSION,
                                     NULL,
                                     acceptTypes,
                                     flags, 0))) {
                
        LOG.error("HttpTransport: HttpOpenRequest error");
        goto exit;
    }


    //
    // Prepares headers
    //
    WCHAR headers[512];
    const char* msg = "download attachment";
    contentLength = strlen(msg);

    const void* msgToSend = (const void*)msg;
    wsprintf(headers, TEXT("Content-Type: %s\r\nContent-Length: %d"), SYNCML_CONTENT_TYPE, contentLength);

    // Timeout to receive a rensponse from server (default = 2 min).
    DWORD timeoutMsec = 2*1000; // 2 minutes
    InternetSetOption(request, INTERNET_OPTION_RECEIVE_TIMEOUT, &timeoutMsec, sizeof(DWORD));
    
    //
    // Try MAX_RETRIES times to send http request, in case of network errors
    //
    DWORD errorCode = 0;
    int numretries;
    for (numretries=0; numretries < MAX_RETRIES; numretries++) {

        //
        // Send a request to the HTTP server.
        //
        if (!HttpSendRequest(request, headers, wcslen(headers), (LPVOID)msgToSend, contentLength)) {

            errorCode = GetLastError();
            char* tmp = createHttpErrorMessage(errorCode);
            LOG.info("HttpSendRequest error %d: %s", errorCode, tmp);                                    
            delete [] tmp;

            if (errorCode == ERROR_INTERNET_OFFLINE_MODE) {                     // 00002 -> retry
                LOG.debug("Offline mode detected: go-online and retry...");
                WCHAR* wurl = toWideChar(url.fullURL);
                InternetGoOnline(wurl, NULL, NULL);
                delete [] wurl;                
                continue;
            }
            else if (errorCode == ERROR_INTERNET_TIMEOUT ||                     // 12002 -> out code 2007
                     errorCode == ERROR_INTERNET_INCORRECT_HANDLE_STATE) {      // 12019 -> out code 2007
                LOG.debug("%s", "HttpTransport: Network error: the request has timed out -> exit.");
                goto exit;
            }
            else if (errorCode == ERROR_INTERNET_CANNOT_CONNECT) {              // 12029 -> out code 2001
                LOG.debug("%s", "HttpTransport: Network error: the attempt to connect to the server failed -> exit");
                goto exit;
            }
            // Other network error: retry.
            LOG.info("HttpTransport: Network error writing data from client: retry %i time...", numretries + 1);
            continue;
        }

        LOG.debug("%s", "HttpTransport: Message sent");

        //
        // Check the status code.
        //
        size = sizeof(status);
        HttpQueryInfo (request,
                       HTTP_QUERY_STATUS_CODE | HTTP_QUERY_FLAG_NUMBER,
                       (LPDWORD)&status,
                       (LPDWORD)&size,
                       NULL);
        
        // OK: status 200
        if (status == HTTP_STATUS_OK) {
        	LOG.debug("HttpTransport: Data sent successfully to server. Server responds OK");
            break;
        }
        else if (status == HTTP_ERROR) {                    // 400 bad request error. retry to send the message
            LOG.info("NHttpTransport:etwork error in server receiving data. "
                     "Server responds 400: retry %i time...", numretries + 1);
            continue;
        }
        else if (status == HTTP_STATUS_SERVER_ERROR ) {     // 500 -> out code 2052
            LOG.debug("%s", "HttpTransport: HTTP server error: 500. Server failure.");
            goto exit;
        }                                        
        else if (status == HTTP_STATUS_NOT_FOUND) {         // 404 -> out code 2060
            LOG.debug("HttpTransport: HTTP request error: resource not found (status %d)", status);
            goto exit;
        }
        else {
            // Other HTTP errors -> OUT
            DWORD code = GetLastError();
            char* tmp = createHttpErrorMessage(code);
            LOG.debug("HttpTransport: HTTP request error: status received = %d): %s (code %d)", status, tmp, code);
		    delete [] tmp;
		    goto exit;
        }
    } // for(numretries = 0; numretries < MAX_RETRIES; numretries++)
    

    // Too much retries -> exit
    if (numretries == MAX_RETRIES) {                        // Network error -> out code 2001
        LOG.error("HttpTransport: HTTP request error: %d attempts failed.", numretries);
        goto exit;
    }
/*
    MessageBox (NULL, TEXT("m"), TEXT("title"), MB_SETFOREGROUND | MB_OK);
    DWORD d = 2048;
    wchar_t bb[2048];
    HttpQueryInfo (request,
                       HTTP_QUERY_RAW_HEADERS_CRLF,
                       (LPVOID)bb,
                       &d,
                       NULL);
    HttpQueryInfo (request,
                       HTTP_QUERY_CONTENT_LENGTH | HTTP_QUERY_FLAG_NUMBER,
                       (LPDWORD)&status,
                       (LPDWORD)&size,
                       NULL);

*/
    //Initialize response
   
//
// ====================================== Reading Response ======================================
//
/*    
    FILE *f = fopen("\\tmp.bin", "wb+");    
    int written = 0;
    ULONG        cbWritten;
    
    do {
        if (!InternetReadFile(request, (LPVOID)bufferA, readBufferSize, &read)) {
            DWORD code = GetLastError();
            char* tmp = createHttpErrorMessage(code);
            LOG.error("InternetReadFile Error: %d - %s", code, tmp);
            delete [] tmp;
            goto exit;
		}       
        written = 0;
        if (read > 0) {
            written = fwrite(bufferA, sizeof(char), read, f);
            //hr = pStream->Write(bufferA, read, &cbWritten);
            realResponseLenght += read;           
        }
        
        if (written != read) {
            LOG.error("Error writing the file");
            goto exit;
        }
        if (exit) {
            LOG.debug("Set to exit");
            goto exit;
        }


    } while (read);
    
    fclose(f);
    
     // free read buffer
    delete [] bufferA; bufferA = NULL;
*/    
    FILE* f                 = NULL;   
    DWORD exitcode          = -1;
    HANDLE downloadThread   = CreateThread(NULL, 0, readWorker, 
                        NULL, 0, NULL);

    if (!downloadThread) {
        LOG.error("Error creating receive thread: code 0x%08x", GetLastError());        
    }
         
    int previousSize = 0;
    bool stop = false;
    while (1) {
        previousSize = totalLenght;

        DWORD waitResult = WaitForSingleObject(downloadThread, 10000);
        switch (waitResult) {

            // Thread exited: socket or Server error occurred -> out
            case WAIT_ABANDONED:
                LOG.debug("receiveThread abandoned");
            case WAIT_OBJECT_0: {                
                GetExitCodeThread(downloadThread, &exitcode);
                LOG.debug("receiveThread ended with code %d", exitcode);                  
                stop = true;
                break;
            }            

            // Some error occurred (case WAIT_FAILED but also WAIT_TIMEOUT)
            // we don't want to continue to downlaod
            default: {
                LOG.debug("Timeout - downloadThread will now be terminated");                
                if (previousSize == totalLenght) {               
                    stop = true; 
                    for (int i = 0; i < 5; i++) {
                        BOOL term = TerminateThread(downloadThread, 1);   
                        LOG.debug("Terminate thread");
                        if (term) {                            
                            break;
                        } else {    
                            Sleep(1000);                            
                            fclose(f);                            
                        }
                    }
                }                
                break;
            }
        }
        if (stop) {
            break;
        }        
    }
    CloseHandle(downloadThread);

    if (exitcode == 0) { // all ok
        // copy the value of the file inside the stream
        f = fileOpen(tmpFile, "rb"); 

        if (f == NULL) {
            LOG.debug("File is null, goto exit");
            goto exit;
        }

        ULONG        cbReadSize;
        const ULONG  ChunkSize = 4096;
        BYTE         Buffer[ChunkSize];
        HRESULT hr;
        do {
            cbReadSize = fread(Buffer, sizeof(BYTE), ChunkSize, f);
            if (cbReadSize > 0)    {
                hr = pStream->Write(Buffer, cbReadSize, &cbWritten);
            }
        } while (!FAILED(hr) && (cbReadSize == ChunkSize));
        
        fclose(f);        
        ret = true;
    } else {
        LOG.error("Thread of downloading the attachment finished in wrong way");                                  
    }
    
    //------------------------------------------------------------- Response read
 
exit:
    // Close the Internet handles.
    if (inet) {
        InternetCloseHandle (inet);
    }
    if (connection) {
        InternetCloseHandle (connection);
    }
    if (request) {
        InternetCloseHandle (request);
    }
    
    if (wurlHost)     delete [] wurlHost;
    if (wurlResource) delete [] wurlResource;
    if (bufferA)      delete [] bufferA;
    
    if (f) {
        fclose(f);
    }
    wchar_t* tt = toWideChar(tmpFile);
    DeleteFile(tt);
    delete [] tt;
    delete [] tmpFile;

    return ret;
}

DWORD WINAPI readWorker(LPVOID lpv) {

    f = fileOpen(tmpFile, "wb+");         
    DWORD read = 0;
    int ret = 0;
   do {        
        if (!InternetReadFile(request, (LPVOID)bufferA, readBufferSize, &read)) {
            DWORD code = GetLastError();
            LOG.error("InternetReadFile Error: %d", code);
            ret = 1;
            goto exit;
		}       
        written = 0;
        
        if (read > 0) {
            written = fwrite(bufferA, sizeof(char), read, f);    
            totalLenght += written;
            if (attachSize > 0) {                 
                SendMessage(h, ID_MYMSG_GO_STEP_PROGRESS, 1, 0); 
            }
        }
        
        if (written != read) {
            LOG.error("Error writing the file");
            ret = 2;
            goto exit;
        }
        LOG.error("read: %i", totalLenght); 

    } while (read);

exit:
    fclose(f);
     // free read buffer
    delete [] bufferA; bufferA = NULL;
    return ret;

}






/**
 * Utility function to retrieve the correspondant message for the Wininet error code passed.
 * Pointer returned is allocated new, must be freed by caller.
 * @param errorCode  the code of the last error
 * @return           the error message for the passed code, new allocated buffer
 */
char* HttpTransport::createHttpErrorMessage(DWORD errorCode) {

    WCHAR* errorMessage = new WCHAR[512];
    memset(errorMessage, 0, 512);

    FormatMessage(
                FORMAT_MESSAGE_FROM_HMODULE,
                GetModuleHandle(L"wininet.dll"),
                errorCode,
                MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
                errorMessage,
                512,
                NULL);

    if (!errorMessage || wcslen(errorMessage) == 0) {
        wsprintf(errorMessage, L"Unknown error.");
    }

    char* ret = toMultibyte(errorMessage);
    if (errorMessage) delete [] errorMessage;
    return ret;
}

